import yaml

from mercury_sdk.mcli import output


def read_config(path):
    try:
        with open(path) as fp:
            return yaml.safe_load(fp)
    except OSError as os_error:
        output.print_and_exit("Error opening or reading file: {}".format(os_error))
    except ValueError as v_error:
        output.print_and_exit("Error deserializing YAML file: {}".format(v_error))



import logging
import sys

import colorama
import pkg_resources

FORMAT = '%(message)s'
program_version = pkg_resources.get_distribution('mercury-sdk').version
LOG = logging.getLogger(__name__)


def setup_logging(verbosity):
    verbosity_map = {
        0: logging.ERROR,
        1: logging.WARNING,
        2: logging.INFO,
        3: logging.DEBUG
    }

    logging.basicConfig(format=FORMAT,
                        level=verbosity_map.get(verbosity, logging.DEBUG))


# Remove fstrings for python < 3.6 compatibility
# def print_basic_info(configuration):
#     LOG.warning(f'{format_version()}\n'
#                 f'Configuration file: {colorama.Fore.MAGENTA}'
#                 f'{configuration.get("config_file")}'
#                 f'{colorama.Style.RESET_ALL}\n'
#                 f'Mercury URL: '
#                 f'{configuration.get("mercury_url")}')
#
#
# def format_version():
#      return f'SDK Version: {colorama.Fore.GREEN}{program_version}' \
#             f'{colorama.Style.RESET_ALL}'


def print_basic_info(configuration):
    LOG.warning('{}\n'
                'Configuration file: {}'
                '{}'
                '{}\n'
                'Mercury URL: '
                '{}'.format(
                            format_version(),
                            colorama.Fore.MAGENTA,
                            configuration.get('config_file'),
                            colorama.Style.RESET_ALL,
                            configuration.get('mercury_url')))


def format_version():
    return 'SDK Version: {}{} {}'.format(
        colorama.Fore.GREEN, program_version, colorama.Style.RESET_ALL)


def print_and_exit(message, code=0):
    print(message)
    sys.exit(code)


def print_rpc_capabilities(active_data):
    for name, capability in active_data['capabilities'].items():
        print('{}{}'.format(colorama.Fore.MAGENTA, name))
        print(colorama.Style.RESET_ALL)
        print('{}'.format(capability['description']))
        print('{}'.format(capability['doc']))




from mercury_sdk.rpc import job

from mercury_sdk.mcli import operations
from mercury_sdk.mcli import press


def press_static_preprocessor(rpc_client, inv_client, template, query, hostname):
    assets = press.build_press_asset_db_from_inv(inv_client, query, hostname)

    with open(template) as fp:
        template_data = fp.read()

    instruction = {
        'assets': assets,
        'template': template_data.splitlines()
    }

    preprocessor = job.Preprocessor(
        rpc_client, operations.de(query), 'press_static_assets', instruction)

    preprocessor.start()
    print(operations.json_format(preprocessor.tasks))
    preprocessor.join()
    print(operations.json_format(preprocessor.status))



import json
from mercury_sdk.http import inventory, rpc
from mercury_sdk.rpc import job
from mercury_sdk.mcli import output


def get_inventory_client(configuration, token=None):
    return inventory.InventoryComputers(
        configuration['mercury_url'],
        max_items=configuration.get('max_items', 250),
        auth_token=token
    )


def json_format(data):
    return json.dumps(data, indent=2)


def make_query(client, q, projection, limit, strip_empty):
    return client.query(
        q, projection=projection, params={"limit": limit}, strip_empty_elements=strip_empty)


def de(data):
    try:
        return json.loads(data)
    except ValueError:
        output.print_and_exit('Query is not valid json', 1)
        return


def query_inventory(client, configuration):
    query = de(configuration['query'])
    if configuration.get('active'):
        query.update({'active': {'$ne': None}})

    return make_query(
        client,
        query,
        projection=configuration['projection'].split(','),
        limit=configuration['max_items'],
        strip_empty=True)


def get_inventory(client, configuration):
    data = client.get(configuration['mercury_id'],
                      projection=configuration['projection'].split(','))
    return json.dumps(data, indent=2)


def get_rpc_client(configuration, token=None):
    return rpc.JobInterfaceBase(
        configuration['mercury_url'],
        auth_token=token)


def make_rpc(client, target_query, method, job_args, job_kwargs, wait=False):
    _job = job.SimpleJob(client, target_query, method, job_args, job_kwargs)
    _job.start()

    if not wait:
        return json.dumps({
            'job_id': _job.job_id,
            'targets': _job.targets
        }, indent=2)

    # Blocks and waits for RPC task to complete
    _job.join(poll_interval=1)

    return json.dumps(_job.tasks, indent=2)


def get_job(client, job_id):
    return json.dumps(client.get(job_id), indent=2)


def get_status(client, job_id):
    return json.dumps(client.status(job_id), indent=2)


def get_tasks(client, job_id):
    return json.dumps(client.tasks(job_id), indent=2)



import json

import yaml

from mercury_sdk.mcli import operations
from mercury_sdk.mcli import output
from mercury_sdk.rpc import job


def configuration_from_yaml(filename):
    """Loads a YAML configuration file.

    :param filename: The filename of the file to load.
    :returns: dict -- A dictionary representing the YAML configuration file
        loaded. If the file can't be loaded, then the empty dict is returned.
    """
    with open(filename) as infile:
        return yaml.load(infile.read())


def press_server(client, target_query, configuration, wait=False):
    try:
        _job = job.SimpleJob(client, target_query, 'press',
                             job_kwargs={
                                 'configuration': configuration_from_yaml(
                                     configuration)})
    except (IOError, OSError) as e:
        output.print_and_exit(
            'Could not load configuration file: {}'.format(e), 1)
        return
    _job.start()

    if not wait:
        return json.dumps({
            'job_id': _job.job_id,
            'targets': _job.targets
        }, indent=2)

    _job.join(poll_interval=1)

    return json.dumps(_job.tasks, indent=2)


def build_press_asset_db_from_inv(client, query, hostname_template):
    """ Simply adds the current ip to the asset store """

    projection = ["mercury_id", "interfaces"]
    assets = {}
    q = operations.de(query)
    q.update({'active': {'$ne': None}})
    data = operations.make_query(client, q, projection, limit=250, strip_empty=True)

    if not data["items"]:
        operations.output.print_and_exit("Query did not match any active targets")

    counter = 0
    for device in data['items']:
        for interface in device['interfaces']:
            if interface['address_info']:
                counter += 1
                assets[device['mercury_id']] = {
                    'snet_ip_address': interface['address_info'][0]['addr'],
                    'hostname': hostname_template.format(**dict(cnt=counter)),
                    'predictable_names': interface['predictable_names']
                }
                break
    return assets



from mercury_sdk.mcli import operations


def build_ansible_inventory(client, query, user='root', hostname=''):
    projection = ["mercury_id", "interfaces"]
    ansible_inventory = []
    q = operations.de(query)
    data = operations.make_query(client, q, projection, limit=250, strip_empty=True)

    if not data["items"]:
        operations.output.print_and_exit("Query did not match any active targets")

    counter = 0
    for device in data['items']:
        for interface in device['interfaces']:
            if interface['address_info']:
                counter += 1
                ansible_inventory.append(
                    "{} ansible_ssh_user={}{}".format(
                        interface['address_info'][0]['addr'],
                        user,
                        ' unique_hostname={}'.format(hostname.format(
                            **dict(cnt=counter)) if hostname else '')
                    ))
                break
    return "\n".join(ansible_inventory)




import logging
import importlib
import os
import yaml

from datetime import datetime, timezone
import dateutil.parser

LOG = logging.getLogger(__name__)
# TODO: Support multiple auth handlers
__auth_handler = None


def write_token(token_data, path):
    with open(path, 'w') as token_file:
        yaml.safe_dump(token_data, stream=token_file)


def read_token(path):
    with open(path) as fp:
        return yaml.safe_load(fp)


# TODO: learn handler names but searching the auth directory
def get_auth_handler(handler_name):
    global __auth_handler
    if not __auth_handler:
        __auth_handler = importlib.import_module(
            '.mcli.auth.{}'.format(handler_name), package='mercury_sdk')
    return __auth_handler


# Since the configuration file is an argument, and handler_name is learned
# from the configuration, this method cannot be used. Once the change is in
# to discover handlers via directory scan, the handler can then add it's own
# argparse arguments. As of now, this isn't used.
def update_parser(handler_name, parser):
    _h = get_auth_handler(handler_name)
    if hasattr(_h, 'add_arguments'):
        _h.add_arguments(parser)


# TODO: index tokens by mercury url
def get_token(configuration, token_path):
    if os.path.exists(token_path):
        LOG.debug('Token exists, checking...')
        token_data = read_token(token_path)
        if datetime.now(timezone.utc) < dateutil.parser.parse(
                token_data['expires_at']):
            return token_data
        else:
            LOG.info('Token expired at: {}'.format(token_data['expires_at']))
            os.remove(token_path)

    LOG.info('Auth handler: {}'.format(configuration['auth_handler']))
    auth_handler = get_auth_handler(configuration['auth_handler'])

    token_data = auth_handler.authenticate(configuration)
    if not configuration.get('no_store'):
        LOG.debug('Writing token file: {}'.format(token_path))
        write_token(token_data, token_path)
    return token_data


def invalidate_token(configuration, token_path):
    if os.path.exists(token_path):
        token_data = read_token(token_path)
        auth_handler = get_auth_handler(configuration['auth_handler'])
        auth_handler.invalidate(configuration, token_data)
        os.remove(token_path)



import logging
import os
import yaml

from mercury_sdk.mcli import configuration

log = logging.getLogger(__name__)


def get_keys_from_file(path):
    with open(os.path.expanduser(path)) as fp:
        return [line.strip() for line in fp.readlines()]


def ssh_authorized_keys(segment, config):
    """

    :param segment:
    :param config:
    :return:
    """
    if 'ssh_authorized_keys' not in segment:
        return

    keys = []
    if 'keys' not in config['modules']:
        config['modules']['keys'] = keys

    key_defs = segment['ssh_authorized_keys']
    for key_def in key_defs:
        if 'key.filename' in key_def:
            log.info('Reading keys from %s', key_def['key.filename'])
            keys += get_keys_from_file(key_def['key.filename'])
        elif 'key.github' in key_def:
            log.info('This is really cool but not implemented yet')
        else:
            log.error("Garbage in ssh_authorized_keys")


def update_vars(segment, config):
    """

    :param segment:
    :param config:
    :return:
    """
    if 'vars' not in segment:
        return
    # vars are overridden with each call
    config['vars'].update(segment['vars'])


def update_segment(segment, config):
    ssh_authorized_keys(segment, config)
    update_vars(segment, config)


def parse_deploy_configuration(path):
    data = configuration.read_config(path)

    deploy_config = {
        'vars': {},
        'modules': {},
        'secrets': {}
    }

    if 'global' in data:
        update_segment(data['global'], deploy_config)

    # make this update_groups
    if 'groups' in data:
        for group in data['groups']:
            # perform query to get targets
            pass









import logging
# TODO: modify cmd2 to support this use case
# import cmd2

import colorama
from mercury_sdk.rpc.job import SimpleJob

LOG = logging.getLogger(__name__)
# PROMPT = f'{colorama.Style.BRIGHT}{colorama.Fore.LIGHTBLUE_EX}(♀)︎{colorama.Fore.MAGENTA}~>' \
#          f'{colorama.Style.RESET_ALL} '

PROMPT = '{}{}(♀)︎{}~>{} '.format(
    colorama.Style.BRIGHT,
    colorama.Fore.LIGHTBLUE_EX,
    colorama.Fore.MAGENTA,
    colorama.Style.RESET_ALL)


class MercuryShell:
    """ Ridiculously simple shell (no readline support) """
    def __init__(self, rpc_client, prompt=PROMPT, initial_query=None, raw=False, quiet=False):
        self.rpc_client = rpc_client
        self.prompt = prompt
        self.query = initial_query
        self.raw = raw
        self.quiet = quiet

    def run_job(self, instruction):
        instruction = 'bash -c "{}"'.format(instruction)
        s = SimpleJob(self.rpc_client, self.query, 'run',
                      job_args=[instruction])
        s.start()
        s.join(poll_interval=.2)
        for t in s.tasks['tasks']:
            if not self.quiet:
                if not self.raw:
                    co = colorama.Fore.CYAN if t['message']['returncode'] == 0 \
                        else colorama.Fore.LIGHTRED_EX
                    # print(f'{co}{t["mercury_id"]}{colorama.Style.RESET_ALL}\n')
                    print('{}{}{}\n'.format(co, t["mercury_id"], colorama.Style.RESET_ALL))
                stdout = t['message']['stdout']
                if stdout:
                    print(stdout)
                stderr = t['message']['stderr']
                if stderr:
                    print(stderr)

    def input_loop(self):
        while True:
            try:
                instruction = input(self.prompt)
            except KeyboardInterrupt:
                print()
                continue
            except EOFError:
                print()
                break

            if not instruction:
                continue

            if instruction == 'exit':
                break

            if instruction.strip()[0] == '!':
                if not len(instruction) > 1:
                    print('Shell command missing')
                print('THIS IS A SHELL ESCAPE: {}'.format(instruction[1:]))
                continue

            self.run_job(instruction)


if __name__ == '__main__':
    from mercury_sdk.http.rpc import JobInterfaceBase
    ib = JobInterfaceBase('http://localhost:9005')
    _s = MercuryShell(ib, initial_query={})
    _s.input_loop()



import argparse
import colorama
import json
import logging
import os
import pkg_resources

from mercury_sdk.mcli import ansible
from mercury_sdk.mcli import operations
from mercury_sdk.mcli import output
from mercury_sdk.mcli import press
from mercury_sdk.mcli.deploy import static_assets
from mercury_sdk.mcli import shell
from mercury_sdk.mcli.auth import auth as mcli_auth
from mercury_sdk.mcli.configuration import read_config

LOG = logging.getLogger(__name__)

COMMANDS = [
    'inventory',
    'rpc',
    'shell'
]

DEFAULT_PROGRAM_DIR = os.path.expanduser('~/.mercury-sdk')
DEFAULT_CONFIG = os.path.join(DEFAULT_PROGRAM_DIR, 'mcli.yml')
TOKEN_CACHE = os.path.join(DEFAULT_PROGRAM_DIR, '.token.yml')

MERCURY_API_URL_ENV = 'MERCURY_API_URL'
MERCURY_API_USERNAME_ENV = 'MERCURY_API_USERNAME'
MERCURY_API_QUERY_ENV = 'MERCURY_API_QUERY'

program_version = pkg_resources.get_distribution('mercury-sdk').version

query_help = """A mercury query to execute in valid JSON.
Use "-" and the value will be read from 
stdout use "@filename" and the query will be 
read from this file
"""


def options():
    parser = argparse.ArgumentParser(
        description='The Mercury Command Line Interface',
        epilog='SDK version {}'.format(program_version))
    parser.add_argument('--version',
                        action='store_true', help='Display the program version')
    parser.add_argument('-c', '--config-file',
                        default=DEFAULT_CONFIG,
                        help='SDK configuration file')
    parser.add_argument('--program-directory', default=DEFAULT_PROGRAM_DIR,
                        help='Alternative location for program data')
    parser.add_argument('--token-cache', default=TOKEN_CACHE,
                        help='alternative location of the token cache')
    parser.add_argument('-m', '--mercury-url',
                        default=os.environ.get(MERCURY_API_URL_ENV),
                        help='The mercury url to use')
    parser.add_argument('-v', dest='verbosity',
                        action='count',
                        default=0,
                        help='Verbosity level -v, somewhat verbose, -vvv '
                             'really verbose')

    parser.add_argument('--no-auth', action='store_true', help='Skip authentication')
    subparsers = parser.add_subparsers(dest='command', help='<command> --help')

    # login
    login_parser = subparsers.add_parser(
        'login',
        help='Login to the authentication service and store the token in the '
             'local environment')

    login_parser.add_argument('--no-store', default=False, action='store_true',
                              help='Do not store the token')
    login_parser.add_argument('-q', '--quiet', action='store_true',
                              help='Do not print the token')

    # logout
    subparsers.add_parser(
        'logout',
        help='Logout of the authentication service')

    # set-token
    set_token_parser = subparsers.add_parser(
        'set-token',
        help='Bypass auth handlers and set a token directly')
    set_token_parser.add_argument('token', help='The token to set')
    set_token_parser.add_argument(
        '--expire-at', default=8,
        help='The number of hours to consider the token valid')

    # inventory
    inv_parser = subparsers.add_parser('inventory',
                                       help='Inventory query operations')
    inv_parser.add_argument('-q', '--query', default='{}',
                            help=query_help)
    inv_parser.add_argument('-p', '--projection', default='',
                            help='Specify the key projection to produce the '
                                 'desired output')
    inv_parser.add_argument('-n', '--max-items', default=100)
    inv_parser.add_argument('-a', '--active',
                            help='Only search for active devices',
                            action='store_true')
    inv_parser.add_argument('mercury_id', default=None, nargs='?',
                            help='Get a device record by mercury_id')

    # rpc
    rpc_parser = subparsers.add_parser(
        'rpc',
        help='RPC commands'
    )
    rpc_subparsers = rpc_parser.add_subparsers(dest='rpc_command')
    rpc_submit_parser = rpc_subparsers.add_parser('submit',
                                                  help='Submit a job')
    rpc_submit_parser.add_argument('-q', '--query',
                                   help=query_help
                                   )
    rpc_submit_parser.add_argument('-t', '--target',
                                   help='A single mercury id to run the command on')
    rpc_submit_parser.add_argument('-m', '--method', help='the RPC to run')
    rpc_submit_parser.add_argument('-a', '--args', nargs='+')
    rpc_submit_parser.add_argument('-k', '--kwargs', default='{}')
    rpc_submit_parser.add_argument('-w', '--wait', action='store_true',
                                   help='Wait for the job to complete and print the results')

    rpc_job_parser = rpc_subparsers.add_parser('job', help='Get information about a job')
    rpc_job_parser.add_argument('job_id', help='A mercury job_id')
    rpc_job_parser.add_argument('-s', '--status', action='store_true',
                                help='Get status information for tasks')
    rpc_job_parser.add_argument('-t', '--tasks', action='store_true',
                                help='Get task data')

    rpc_list_parser = rpc_subparsers.add_parser('list', help='list device capabilities')
    rpc_list_parser.add_argument('target', help='The mercury id of the target')

    # shell
    shell_parser = subparsers.add_parser(
        'shell',
        help='Enter a shell for interactive inventory management')
    shell_parser.add_argument('-q', '--query', default=None,
                              help='Set the initial target query for the shell')
    shell_parser.add_argument('-t', '--target', default=None,
                              help='The mercury_id of a single target')
    shell_parser.add_argument('-r', '--run', default=None,
                              help='Instead of entering the shell,'
                                   'run the command, print the result, and '
                                   'exit')
    shell_parser.add_argument('--quiet', default=False, action='store_true',
                              help='Suppress command output')
    shell_parser.add_argument('--raw', default=False, action='store_true',
                              help='Do not print agent info, only the raw command output')

    # press
    press_parser = subparsers.add_parser('press',
                                         help='Install an operating system to /mnt/press')
    press_parser.add_argument('-c', '--configuration', dest='press_configuration',
                              help='The path to a press configuration in yaml format')
    press_parser.add_argument('-t', '--target', help='The mercury_id of a single target')
    press_parser.add_argument('-w', '--wait', action='store_true',
                              help='Wait for the job to complete and print the results')

    # deployment
    deployment_parser = subparsers.add_parser(
        'deploy',
        help='[EXPERIMENTAL] Deploy many servers at once using an asset file')
    deployment_parser.add_argument(
        '-q', '--query', help=query_help, required=True)
    # deployment_parser.add_argument(
    #     '-a', '--asset-backend', help='The asset backend plugin to use')
    deployment_parser.add_argument('--template', required=True)
    deployment_parser.add_argument("--hostname", default='')

    # ansible helpers
    ansible_parser = subparsers.add_parser(
        'ansible',
        help='Helper commands for ansible'
    )
    ansible_sub_parsers = ansible_parser.add_subparsers(dest="ansible_command")
    ansible_inventory_parser = ansible_sub_parsers.add_parser('inventory')
    ansible_inventory_parser.add_argument("-q", "--query", required=True)
    ansible_inventory_parser.add_argument("-u", "--user", default='root')
    ansible_inventory_parser.add_argument("--hostname-template", default='')

    namespace = parser.parse_args()
    if namespace.version:
        output.print_and_exit(output.format_version(), 0)
    if not namespace.command:
        parser.print_help()
        output.print_and_exit('\nPlease specify a command...')
    return namespace


def merge_configuration(namespace, configuration):
    _program_configuration = namespace.__dict__
    if not namespace.mercury_url:
        _m = configuration.get('mercury_api', {}).get('url')
        if not _m:
            output.print_and_exit('Mercury Service URL is undefined', 1)
        _program_configuration['mercury_url'] = _m
    _program_configuration['auth'] = configuration.get('auth')
    _program_configuration['auth_handler'] = configuration.get('auth_handler')
    return _program_configuration


def router(command, configuration):
    output.print_basic_info(configuration)
    if command == 'login':

        token_data = mcli_auth.get_token(configuration, TOKEN_CACHE)
        if not configuration['quiet']:
            print('Expires: {expires_at}, Token: {token}'.format(**token_data))
        return
    elif command == 'logout':
        mcli_auth.invalidate_token(configuration, TOKEN_CACHE)
        return

    if not configuration['no_auth'] and configuration['auth'] and configuration['auth_handler']:
        token = mcli_auth.get_token(configuration, TOKEN_CACHE)['token']
    else:
        token = None

    if command == 'inventory':
        inv_client = operations.get_inventory_client(configuration, token)
        if configuration.get('mercury_id'):
            print(operations.get_inventory(inv_client, configuration))
        else:
            print(operations.json_format(operations.query_inventory(inv_client, configuration)))

    def _prepare_rpc():
        _rpc_client = operations.get_rpc_client(configuration, token)
        query = configuration.get('query')
        target = configuration.get('target')
        if query:
            try:
                _target_query = json.loads(configuration['query'])
            except ValueError:
                output.print_and_exit('query is not valid JSON')
                return
        elif target:
            _target_query = {'mercury_id': target.strip()}
        else:
            output.print_and_exit('Must provide a query or target')
            return

        return _rpc_client, _target_query

    if command == 'rpc':
        rpc_command = configuration['rpc_command']
        if rpc_command == 'submit':
            rpc_client, target_query = _prepare_rpc()
            kwargs = json.loads(configuration['kwargs'])
            print(operations.make_rpc(rpc_client,
                                      target_query,
                                      configuration['method'],
                                      configuration['args'],
                                      kwargs,
                                      wait=configuration.get('wait')))
        if rpc_command == 'job':
            if configuration['tasks'] and configuration['status']:
                output.print_and_exit('--tasks cannot be combined with --status')

            rpc_client = operations.get_rpc_client(configuration, token)

            if configuration['tasks']:
                data = operations.get_tasks(rpc_client, configuration['job_id'])
            elif configuration['status']:
                data = operations.get_status(rpc_client, configuration['job_id'])
            else:
                data = operations.get_job(rpc_client, configuration['job_id'])

            print(data)

        if rpc_command == 'list':
            inv_client = operations.get_inventory_client(configuration, token)
            data = inv_client.get(configuration.get('target'), projection=['active'])
            if not data['active']:
                output.print_and_exit('{} is not active'.format(configuration['target']))
            output.print_rpc_capabilities(data['active'])

    if command == 'press':
        rpc_client, target_query = _prepare_rpc()
        print(press.press_server(rpc_client, target_query, configuration['press_configuration']))

    if command == 'shell':
        rpc_client, target_query = _prepare_rpc()

        mshell = shell.MercuryShell(rpc_client, initial_query=target_query,
                                    quiet=configuration['quiet'], raw=configuration['raw'])

        if configuration.get('run'):
            mshell.run_job(configuration.get('run').strip())
        else:
            mshell.input_loop()

    if command == 'ansible':
        ansible_command = configuration['ansible_command']
        if ansible_command == 'inventory':
            inv_client = operations.get_inventory_client(configuration, token)
            print(ansible.inventory.build_ansible_inventory(
                inv_client,
                configuration['query'],
                configuration['user'],
                configuration['hostname_template']))

    if command == 'deploy':
        inv_client = operations.get_inventory_client(configuration, token)
        rpc_client, target_query = _prepare_rpc()

        # print(operations.json_format(press.build_press_asset_db_from_inv(
        #     inv_client, configuration['query'], configuration['hostname'])))

        static_assets.press_static_preprocessor(
            rpc_client, inv_client, configuration['template'], configuration['query'],
            configuration['hostname'])


def main():
    colorama.init(autoreset=True)
    namespace = options()

    if not os.path.exists(namespace.program_directory):
        os.makedirs(namespace.program_directory, 0o700)

    if os.path.isfile(namespace.config_file):
        configuration = read_config(namespace.config_file)
    else:
        configuration = {}

    output.setup_logging(namespace.verbosity)

    program_configuration = merge_configuration(namespace, configuration)

    router(namespace.command, program_configuration)



import getpass
import os
import requests

KEYSTONE_USER_ENV = 'KEYSTONE_USER'
KEYSTONE_API_URL_ENV = 'KEYSTONE_API_URL'


def add_arguments(login_parser):
    login_parser.add_argument('--keystone-api-url',
                              default=os.environ.get(KEYSTONE_API_URL_ENV),
                              help='[keystone plugin] keystone url to use')
    login_parser.add_argument('--keystone-user',
                              default=os.environ.get(KEYSTONE_USER_ENV),
                              help='[keystone plugin] keystone user to use')


def authenticate(configuration):
    auth_config = configuration['auth']
    _payload = {
        'auth': {}
    }
    if auth_config['type'] == 'password':
        _payload['auth']['passwordCredentials'] = {
            'username': (os.environ.get(KEYSTONE_USER_ENV) or
                         auth_config['username']),
            'password': auth_config.get('password') or getpass.getpass()
        }
    elif auth_config['type'] == 'rax-rsa':
        _payload['auth']['RAX-AUTH:rsaCredentials'] = {
            'username': auth_config['username'],
            'tokenKey': getpass.getpass('PIN + RSA: ')
        }
    else:
        raise Exception('Auth type is not supported')

    if 'domain' in auth_config:
        _payload['auth'][auth_config['domain']['key']] = {
            'name': auth_config['domain']['name']
        }

    response = requests.post(
        os.environ.get(KEYSTONE_API_URL_ENV) or auth_config['url'],
        json=_payload)

    if not response.ok:
        raise Exception('Error authenticating : {}'.format(
            response.status_code))

    response = response.json()

    return {
        'token': response['access']['token']['id'],
        'expires_at': response['access']['token']['expires']
    }


def invalidate(configuration, token_data):
    pass
